/**
 * Adds a slider to the tutorial videos, if the list contains more than 3 tutorial videos.
 *
 * The .shopUiVideoslider() adds a slider to the tutorial videos on the help page and enables the user to navigate between videos.
 *
 * ### Examples
 * Binds the shopUiVideoslider()-Dialog to the following <div>, if the element contains more than 3 tutorial videos (see corresponding TLE) and calls the open-method of the widget attached to the page element.
 *
 * HTML:
 * <div class="ep-helpcenter-video-track">
 *
 * JavaScript:
 *  <script>
 *    require([
 *      'jquery',
 *      'de_epages/shop/ui/videoslider',
 *
 *      '$ready!'
 *    ], function($, de_epages) {
 *      $('.ep-helpcenter-video-list').shopUiVideoslider(
 *          setWidthToParentElement: true
 *      );
 *  });
 *  </script>
 *
 *
 *
 * @class jQuery.ui.shopUiVideoslider
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses jquery/slick/slick
 * @uses jQuery.ui.widget
 * @uses de_epages/shop/ui/videoslider.js
 * @since 7.35.0
 */

 /**
 * @cfg {Integer} [slidesToShow] set number of slides to show
 */

/**
 * @cfg {Integer} [slidesToScroll] set number of slides to scroll
 */

/**
 * @cfg {Boolean} [setWidthToParentElement] set width of the slider to the width of the parent element
 */

/**
 * See `jQuery.ui.shopUiVideoslider` for details.
 *
 * @param {String} [slidesToShow] set number of slides to show
 * @param {Integer} [slidesToScroll] set number of slides to scroll
 * @param {Boolean} [setWidthToParentElement] set width of the slider to the width of the parent element
 *
 * @method shopUiVideoslider
 * @member jQuery
 *
 *
 * @since 7.35.0
 */

/*
 * @copyright       © Copyright 2006-2019, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/shop/ui/videoslider
 */

define("de_epages/shop/ui/videoslider", [
    "jquery",
    "ep",
    "de_epages",

    "jquery/slick/slick",
    "jquery/ui/widget"
], function($, ep, de_epages) {
    $.widget('ui.shopUiVideoslider', {
        options: {
            slidesToShow: 3,
            slidesToScroll: 3,
            setWidthToParentElement: false
        },

        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.shopUiVideoslider
         *
         * @since 7.35.0
         */
        _create: function() {
            var self = this,
                o = self.options,
                resizeId;

            // Calculate slider track width from parent element
            if(o.setWidthToParentElement) {
                self._setWidth();

                // recalculate slider width, when no resize event has been fired for 150 ms
                $(window).on('resize.uiVideoslider', function() {
                    clearTimeout(resizeId);
                    resizeId = setTimeout($.proxy(self, '_hResize'), 150);
                });
            }

            self._initSlider();
        },

        /**
         * Method is called from _create(), only when window gets resized
         *
         * @member jQuery.ui.shopUiVideoslider
         *
         * @since 7.35.0
         */

        _hResize: function() {
            var self = this,
                elem = self.element;

            // reinitialize slider to keep accurate slider width
            elem.slick('unslick');
            elem[0].style.width = "";
            self._setWidth();
            self._initSlider();
        },

        /**
         * Method is once called during initialization and during resize
         *
         * @member jQuery.ui.shopUiVideoslider
         *
         * @since 7.35.0
         */
        _setWidth: function() {
            var self = this,
                elem = self.element;

            elem.outerWidth(elem.parent().outerWidth());
        },

        /**
         * Method to initialize slick slider
         *
         * @member jQuery.ui.shopUiVideoslider
         *
         * @since 7.35.0
         */
        _initSlider: function() {
            var self = this,
                elem = self.element,
                o = self.options;

            elem.on('init', function(event, slick) {
                // show/hide arrows (depending on slick variables, see: https://github.com/kenwheeler/slick/)
                self._setArrows(slick);
            });

            // Initialize Slick Slider
            // (Config param: https://github.com/kenwheeler/slick/)
            elem.slick({
                infinite: false,
                slidesToShow: o.slidesToShow,
                slidesToScroll: o.slidesToScroll,
                draggable: false,
                prevArrow: '<span class="ep-slick-prev fa fa-chevron-circle-left"></span>',
                nextArrow: '<span class="ep-slick-next fa fa-chevron-circle-right"></span>',
            });

            elem.on('afterChange', function(event, slick, currentSlide) {
                // show/hide arrows (depending on slick variables, see: https://github.com/kenwheeler/slick/)
                self._setArrows(slick);
            });
        },

        /**
         * Method to set slider arrow elements.
         *
         * @member jQuery.ui.shopUiVideoslider
         *
         * @since 7.35.0
         */
        _setArrows: function(slick) {
            // if right border is reached => Hide arrow-element
            slick.$nextArrow[(slick.slideCount - slick.currentSlide <= slick.options.slidesToShow) ? 'hide' : 'show']();
            // if left border is reached => Hide arrow-element
            slick.$prevArrow[(slick.currentSlide === 0) ? 'hide' : 'show']();
        },

        destroy: function() {
            var self = this,
                elem = self.element;

            // remove resize event
            $(window).off('resize.uiVideoslider');

            // destroy slider plugin.
            elem.slick('unslick');

            this._superApply(arguments);
        }
    });

    return de_epages;
});