/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          ep.fn.busy
 */

define("ep/fn/busy", [
    "jquery",
    "ep"
], function ($, ep) {

    var stack = [];

    // remove all busy layers on page leave
    $(window)
        .on("unload", function(){
            $(stack)
                .remove();
            stack = [];
        });

    ep.fn.extend({
        /**
         * Add or remove a busy layer to each of the set of matched elements.
         *
         * The `.busy()` method adds/removes a busy layer to/from each of the set of matched elements.
         *
         * On page unload every busy layer will be removed. This prevents showing busy layer after
         * come back via browsers back button.
         *
         * ### Examples
         * Show a busy layer on body.
         *
         * JavaScript:
         *
         *     ep('div').busy('show');
         *
         * HTML:
         *
         *     <body>
         *       <h1>Example</h1>
         *       <div>My test case ...</div>
         *       <div>A footer.</div>
         *     </body>
         *
         * Results:
         *
         *     <body>
         *       <h1>Example</h1>
         *       <div>My test case ...</div>
         *       <div>A footer.</div>
         *       <div class="ep-busy"></div>
         *     </body>
         *
         * Hide a busy layer on body.
         *
         * JavaScript:
         *
         *     ep('div').busy('hide');
         *
         * HTML:
         *
         *     <body>
         *       <h1>Example</h1>
         *       <div>My test case ...</div>
         *       <div>A footer.</div>
         *       <div class="ep-busy"></div>
         *     </body>
         *
         * Results:
         *
         *     <body>
         *       <h1>Example</h1>
         *       <div>My test case ...</div>
         *       <div>A footer.</div>
         *     </body>
         *
         *
         * @param {String} action An action for busy layer, 'show' or 'hide'.
         *
         * @method busy
         * @member jQuery
         *
         * @since 6.11.0
         */
        busy: function( action, options ){
            var type = options && options.type ? options.type : undefined;

            return this.each(function () {
                var elem = $(this),
                layer = elem.find('> .ep-busy');

                if (action === 'hide') {
                    var pos = $.data(this, 'epBusyOldposition');

                    if (pos === 'static') {
                        $.removeData(this, 'epBusyOldposition');
                        elem.css('position', 'relative');
                    }

                    layer.remove();
                } else if (!layer.length) {
                    var pos = elem.css('position'),
                        cssSettings = {
                            'border-radius': elem.css('border-radius')
                        };

                    if (pos === 'static') {
                        $.data(this, 'ep-busy-oldposition', pos);
                        elem.css('position', 'relative');
                    }

                    layer = $('<div>');


                    stack.push(
                        layer.attr('class', 'ep-busy')
                            .css(cssSettings)
                            .appendTo(elem)[0]
                    );

                    if (type === 'dynamic') {
                        var timeout,
                        /** calculate the center of the visible part of the element and position the loading image accordingly
                        /*  fall back to 50% if it is not within bounds at all
                        **/
                            centerInput = function() {
                            var elemRect = elem.get(0).getBoundingClientRect(),
                                horizontalCenter = (elemRect.x > window.innerWidth || elemRect.x < 0)
                                    ? '50%'
                                    : (Math.min(elemRect.x + elemRect.width, window.innerWidth) + Math.max(elemRect.x, 0)) / 2 + 'px',
                                verticalCenter = (elemRect.y > window.innerHeight || elemRect.y < 0)
                                    ? '50%'
                                    : (Math.min(elemRect.y + elemRect.height, window.innerHeight) + Math.max(elemRect.y, 0)) / 2 + 'px';

                            cssSettings = $.extend(cssSettings, {
                                'background-attachment': 'fixed',
                                'background-position-x': horizontalCenter,
                                'background-position-y': verticalCenter
                            });

                            layer.css(cssSettings);
                            timeout = undefined;
                        };

                        centerInput();

                        // reset position on scroll / resize
                        $(window).on('scroll resize', function () {
                            if (!timeout) {
                                timeout = setTimeout(centerInput, 50);
                            }
                        });
                    }
                }
            });
        }
    });

    return ep;
});
